@@ -1,12 +1,8 @@
 use std::{fmt::{Debug, Display}, marker::PhantomData, ops::{Deref, DerefMut}};
-#[cfg(not(features = "deadlocks"))]
-use std::sync::{RwLockReadGuard, RwLockWriteGuard};
-#[cfg(features = "deadlocks")]
-use no_deadlocks::{RwLockReadGuard, RwLockWriteGuard};
+use parking_lot::{RwLockReadGuard, RwLockWriteGuard};
 
 /// The resource trait 
 pub trait Resource : Send + Sync + 'static {}
-impl<T : Send + Sync + 'static> Resource for T {}
 
 impl dyn Resource {
     pub(in crate) unsafe fn downcast_ref<T : Resource>(&self) -> &T{
